#include <iostream>
#include <cstdio>
#include <algorithm>
#include <vector>
#include <queue>
#include <ctime>
using namespace std;

//statistics constants
const int BOARD_STEPS = 12;

//my solution constants
const double DEGRADATER_THRESHOLD = 0.0001;
const double REUSED_RATE = 1.2;

//game constants
const int MAX_BOARD = 52;
const int MAX_FORM = 12;
const int MAX_JUMPER = 102;
const double MAX_TIME = 1.7 * CLOCKS_PER_SEC;
const int SEED_CONSTANT = 10831;

struct Jumper
{
    int index, r, c, k, t, form[MAX_FORM][MAX_FORM], sumSq;
};


bool bySmallestDegradator (Jumper const& a, Jumper const& b)
{
    return a.k < b.k ||
              (a.k == b.k && a.sumSq > b.sumSq);
}
bool byBiggestMinSafety (Jumper const& a, Jumper const& b)
{
    return a.t > b.t ||
              (a.t == b.t && a.sumSq > b.sumSq);
}

struct Triple { int index, x, y;};

int upperLimit;
int S;
int allNumSq;

int getNumSquares(Jumper const & jp)
{
    int numSquares = 0;
    for(int i=0; i<jp.r; i++)
        for(int j=0; j<jp.c; j++)
            numSquares += jp.form[i][j];
    return numSquares;
}

void getInput(int & N, int & M, int board[][MAX_BOARD], int & P, Jumper jumper[])
{
    cin >> N >> M >> P;
    for(int i=0; i<N; i++)
        for(int j=0; j<M; j++)
            cin >> board[i][j];

    for(int i=0; i<P; i++)
    {
        cin >> jumper[i].r >> jumper[i].c >> jumper[i].k >> jumper[i].t;
        jumper[i].index = i;
        for(int a=0; a<jumper[i].r; a++)
        {
            char buffer[MAX_FORM];
            cin >> buffer;
            for(int b=0; b<jumper[i].c; b++)
                jumper[i].form[a][b] = buffer[b] - '0';
        }
        jumper[i].sumSq = getNumSquares(jumper[i]);
    }
}

void printInput(int const& N, int const& M, int const board[][MAX_BOARD], int const& P,
                Jumper const jumper[])
{
    //test printing
    cout << N << " " << M << " " << P << endl;
    for(int i=0; i<N; i++)
    {
        for(int j=0; j<M; j++)
            cout << board[i][j] << " ";
        cout << endl;
    }

    for(int i=0; i<P; i++)
    {
        cout << jumper[i].r << " " << jumper[i].c << " " <<
                jumper[i].k << " " << jumper[i].t << " " << jumper[i].index << endl;
        for(int a=0; a<jumper[i].r; a++)
        {
            for(int b=0; b<jumper[i].c; b++)
                cout << jumper[i].form[a][b];
            cout << endl;
        }
    }
}

int getS(int const& N, int const& M, int const board[][MAX_BOARD])
{
    int s = 0;
    for(int i=0; i<N; i++)
        for(int j=0; j<M; j++)
            s += board[i][j];
    return s;
}

int getAllNumSq(int const& P, Jumper const jumper[])
{
    int allNumSq = 0;
    for(int i=0; i < P; i++)
        allNumSq += jumper[i].sumSq;
    return allNumSq;
}

class Greedy
{
    public:
        int N;
        int M;
        int board[MAX_BOARD][MAX_BOARD];
        int P;
        vector<Jumper> jumper;
        vector<Triple> realJumpers;

        int mySum;
        int myNumSq;
        int degradatorBorder;

    Greedy(int const& N, int const& M, int const inBoard[][MAX_BOARD], int const& P,
                vector<Jumper> & inJumper) : N(N), M(M), P(P), mySum(0), myNumSq(0), jumper(inJumper)
    {
        copyBoard(inBoard);

        for(int i=0; i<P; i++)
        {
            int x, y, landSum;
            bool isValid = findBestLanding(jumper[i], x, y, landSum);
            if(isValid)
            {
                addRealJumper(jumper[i].index, x, y);
                degradateLanding(jumper[i], x, y);

                int numSquares = jumper[i].sumSq;
                mySum += landSum;
                myNumSq += numSquares;
            }
        }
    }

    void copyBoard(int const inBoard[][MAX_BOARD])
    {
        for(int i=0; i<N; i++)
            for(int j=0; j<M; j++)
                board[i][j] = inBoard[i][j];
    }

    bool operator<(Greedy const& b) const
    {
        return this->mySum < b.mySum;
    }

    int getSum(Jumper const & jp, int x, int y)
    {
        int sum = 0;
        for(int i=0; i<jp.r; i++)
            for(int j=0; j<jp.c; j++)
                if(jp.form[i][j] == 1)
                {
                    if(board[x+i][y+j] < jp.t)
                        return -1;
                    sum += board[x+i][y+j];
                }

        return sum;
    }

    bool findBestLanding(Jumper const & jp, int & x, int & y, int & maxSum)
    {
        maxSum = 0;
        for(int i=0; i<=N - jp.r; i++)
            for(int j=0; j<=M - jp.c; j++)
            {
                int currSum = getSum(jp, i, j);
                if(currSum > maxSum)
                {
                    maxSum = currSum;
                    x = i;
                    y = j;
                }
            }

        return maxSum > 0;
    }


    void degradateLanding(Jumper const & jp, int x, int y)
    {
        for(int i=0; i<jp.r; i++)
            for(int j=0; j<jp.c; j++)
                if(jp.form[i][j] == 1)
                    board[x+i][y+j] /= jp.k;
    }

    void addRealJumper(int index, int x, int y)
    {
        Triple rjp;
        rjp.index = index;
        rjp.x = x;
        rjp.y = y;

        realJumpers.push_back(rjp);
    }

    void printRealJumpers(bool printTotalSum = false)
    {
        cout << realJumpers.size() << endl;
        for(int i=0; i<realJumpers.size(); i++)
            cout << (realJumpers[i].index + 1) << " " << (realJumpers[i].x + 1)
                 << " " << (realJumpers[i].y + 1) << endl;

        if(printTotalSum)
            cout << endl << "total sum: " << this->mySum << endl;
    }

    int getMaxElement()
    {
        int maxElement = board[0][0];
        for(int i=0; i<N; i++)
            for(int j=0; j<M; j++)
                maxElement = max(maxElement, board[i][j]);
        return maxElement;
    }

    void printStatistics()
    {

        int maxElement = getMaxElement();
        cout << endl
             << "mySum: " << mySum << ", myNumSq: " << myNumSq
             << ", percent: " << (double) mySum / upperLimit
             << ", degJumpers: " << degradatorBorder
             << ", finalMaxElement: " << maxElement << endl;

        cout << "board:" << endl;
        for(int i=0; i<N; i++)
        {
            for(int j=0; j<M; j++)
                cout << board[i][j] << " ";
            cout << endl;
        }

        cout << "steps board: " << endl;
        for(int i=0; i<N; i++)
        {
            for(int j=0; j<M; j++)
                cout << ((board[i][j] * BOARD_STEPS - 1) / maxElement) << " ";
            cout << endl;
        }
    }
};

class Board
{
    public:
        Board(int N, int M, int const inBoard[][MAX_BOARD]) : N(N), M(M)
        {
            for(int i=0; i<MAX_BOARD; i++)
                for(int j=0; j<MAX_BOARD; j++)
                    board[i][j] = inBoard[i][j];
        }


        int getUpperLimit(int const& P, Jumper const jumper[],
                          int & limitReusedSquares, int & maxSquareSafety) const
        {
            priority_queue<pair<int, int> > squaresQueue;
            priority_queue<int, vector<int>, greater<int> > dividersQueue;

            for(int i=0; i<N; i++)
                for(int j=0; j<M; j++)
                    squaresQueue.push(make_pair(board[i][j], 0));

            for(int i=0; i<P; i++)
            {
                int power = jumper[i].sumSq;
                for(int j=0; j<power; j++)
                    dividersQueue.push(jumper[i].k);
            }

            maxSquareSafety = squaresQueue.top().first;
            limitReusedSquares = 1;
            int upperLimit = 0;
            while(!dividersQueue.empty())
            {
        //        cout << "top square: " << squaresQueue.top().first << ", top divider: " << dividersQueue.top() << endl;

                upperLimit += squaresQueue.top().first;
                squaresQueue.push( make_pair(squaresQueue.top().first / dividersQueue.top(), 1));
                limitReusedSquares += squaresQueue.top().second;
                squaresQueue.pop();
                dividersQueue.pop();
            }

            return upperLimit;
        }

        int getJumpSum(Jumper const & jp, int x, int y) const
        {
            int sum = 0;
            for(int i=0; i<jp.r; i++)
                for(int j=0; j<jp.c; j++)
                    if(jp.form[i][j] == 1)
                    {
                        if(x+i >= N || y+j >= M || board[x+i][y+j] < jp.t)
                            return -1;
                        sum += board[x+i][y+j];
                    }

            return sum;
        }

        void degradateLanding(Jumper const & jp, int x, int y)
        {
            for(int i=0; i<jp.r; i++)
                for(int j=0; j<jp.c; j++)
                    if(jp.form[i][j] == 1)
                        board[x+i][y+j] /= jp.k;
        }

    public:
        int N;
        int M;
        int board[MAX_BOARD][MAX_BOARD];
};

void printFirstChoiceDeviationFromUpperLimit(Board inputBoard, Jumper const jumper[], int P, int bestSum)
{
    cout << endl << "first choice deviation from upper limit: " << endl;
    for(int i=0; i<inputBoard.N; i++)
    {
        for(int j=0; j<inputBoard.M; j++)
        {
            Board copyBoard = inputBoard;
            int jumpSum = copyBoard.getJumpSum(jumper[0], i, j);
            copyBoard.degradateLanding(jumper[0], i, j);

            int tmpReusedSquares;
            int tmpSquareSafety;
            int afterJumpUpperLimit = copyBoard.getUpperLimit(P-1, jumper+1,
                                                              tmpReusedSquares, tmpSquareSafety);
            if(jumpSum == -1)
                cout << " - ";
            else if (jumpSum + afterJumpUpperLimit < bestSum)
                cout << " -=- ";
            else
                cout << " " << upperLimit - (jumpSum + afterJumpUpperLimit) << " ";
        }
        cout << endl;
    }
}



int getDegradatorBorder(int limitReusedSquares, int limitDegradatorFactor, int P, vector<Jumper> const& inJumper)
{
    for(int i=0; i<P; i++)
    {
        if(inJumper[i].k <= limitDegradatorFactor && limitReusedSquares > 0)
            limitReusedSquares -= inJumper[i].sumSq;
        else
            return i;
    }
    return P;
}

Greedy getSortingGready(int const& N, int const& M, int const inBoard[][MAX_BOARD], int const& P,
                Jumper const inJumper[], int limitReusedSquares, int maxSquareSafety,
                int reuseModifier = 0)
{
    vector<Jumper> useJumper(inJumper, inJumper + P);

    sort(useJumper.begin(), useJumper.end(), bySmallestDegradator);
    int degradatorBorder = getDegradatorBorder(limitReusedSquares * REUSED_RATE * reuseModifier,
                                               maxSquareSafety / (upperLimit * DEGRADATER_THRESHOLD),
                                               P, useJumper);
    sort(useJumper.begin() + degradatorBorder, useJumper.end(), byBiggestMinSafety);

    return Greedy(N, M, inBoard, P, useJumper);
}

int main()
{
    unsigned stTime = clock();
    freopen("parash.in", "r", stdin);
    freopen("parash.out", "w", stdout);

//    freopen("test0.txt", "r", stdin);
//    freopen("test1.txt", "r", stdin);
//    freopen("parash.01.in", "r", stdin);
//    freopen("parash.02.in", "r", stdin);
//    freopen("parash.03.in", "r", stdin);
//    freopen("parash.04.in", "r", stdin);
//    freopen("parash.05.in", "r", stdin);
//    freopen("parash.02.new.in", "r", stdin);
//    freopen("parash.1.01.in", "r", stdin);
//    freopen("parash.1.02.in", "r", stdin);
//    freopen("parash.1.03.in", "r", stdin);

    int N;
    int M;
    int board[MAX_BOARD][MAX_BOARD];
    int P;
    Jumper jumper[MAX_JUMPER];

    getInput(N, M, board, P, jumper);
    Board inputBoard(N, M, board);

    int limitReusedSquares;
    int maxSquareSafety;
    upperLimit = inputBoard.getUpperLimit(P, jumper, limitReusedSquares, maxSquareSafety);
    S = getS(N, M, board);
    allNumSq = getAllNumSq(P, jumper);

    //cerr << S << endl;

    //greedy (class, and this is it's constructor)
    //check 3 greedies - REUSED_RATE = 0, REUSED_RATE = 1.2, REUSED_RATE = 20000
    Greedy a = getSortingGready(N, M, board, P, jumper, limitReusedSquares, maxSquareSafety, 0);
    Greedy b = getSortingGready(N, M, board, P, jumper, limitReusedSquares, maxSquareSafety, 1);
    Greedy c = getSortingGready(N, M, board, P, jumper, limitReusedSquares, maxSquareSafety, 20000);

    Greedy best = max(a, max(b, c));

    srand (SEED_CONSTANT);
    while(true)
    {
        if(clock() - stTime >= MAX_TIME)
            break;

        vector<Jumper> useJumper(jumper, jumper + P);
        random_shuffle (useJumper.begin(), useJumper.end());
        Greedy x(N, M, board, P, useJumper);
        best = max(best, x);
    }

    best.printRealJumpers();

    //statistical data
//    cout << endl << "my best: " << best.mySum << endl;
//    cout << "time: " << double (clock() - stTime)/CLOCKS_PER_SEC << endl;
//
//    cout << endl << endl;
//    cout << "N: " << N << ", M: " << M << ", P: " << P << ", sq: " << allNumSq << endl;
//    cout << "S: " << S << ", upperLimit: " << upperLimit
//         << ", cover: " << (double) allNumSq/(N*M) << ", limitRation: " << (double) upperLimit/S << endl;
//    cout << "limitReuse: " << limitReusedSquares
//         << ", limitDegradator: " << (int) maxSquareSafety / (upperLimit * DEGRADATER_THRESHOLD) << endl;
//    a.printStatistics();
//    b.printStatistics();
//    c.printStatistics();
//
//    printFirstChoiceDeviationFromUpperLimit(inputBoard, jumper, P, best.mySum);

    return 0;
}
